@@ -15,51 +15,10 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 # OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-import sys, os, jinja2, yaml, argparse, re, copy, getpass, datetime, pytz, traceback
-
-__version__ = '1.9.0'
-jinja2_filters = []
-
-def import_filters(errc = 0):
-  try:
-    from ansible.plugins.filter import core
-    jinja2_filters.append(core.FilterModule().filters())
-  except Exception:
-    print('warning: unable to import ansible \'core\' filters - requires ansible', file=sys.stderr)
-    errc += 1
-      
-  try:
-    import netaddr
-
-    try:
-      from ansible.plugins.filter import ipaddr
-    except Exception:
-      try:
-        from ansible_collections.ansible.netcommon.plugins.filter import ipaddr
-      except Exception:
-        raise Exception()
-
-    filters = {}
-    for k, v in ipaddr.FilterModule().filters().items():
-      filters[k] = v
-      filters['ansible.netcommon.' + k] = v
-
-    jinja2_filters.append(filters)
-
-  except Exception:
-    print('warning: unable to import ansible \'ipaddr\' filter - requires ansible and netaddr', file=sys.stderr)
-    errc += 1
-
-  return errc
-
-
-def __format_bytes(b):
-  for u in [ '', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y' ]:
-    if b >= 1000:
-      b /= 1000
-    else:
-      return '{:.2f}'.format(b).rstrip('0').rstrip('.') + u + 'B'
+import sys, os, io, importlib, argparse, re, copy, getpass, datetime, traceback
+import jinja2, yaml, pytz
 
+__version__ = '1.10.0'
 
 def main():
   try:
@@ -67,12 +26,14 @@ def main():
       print('JinjaFx v' + __version__ + ' - Jinja2 Templating Tool')
       print('Copyright (c) 2020-2022 Chris Mason <chris@netnix.org>\n')
 
-    jinjafx_usage = '(-t <template.j2> [-d <data.csv>] | -dt <dt.yml>) [-g <vars.yml>] [-ed <exts dir>] [-o <output file>] [-od <output dir>] [-m] [-q]'
+    jinjafx_usage = '-t <template.j2> [-d <data.csv>] [-g <vars.yml>] [-ed <exts dir>] [-o <output file>] [-od <output dir>] [-m] [-q]'
+    # jinjafx_usage = '(-t <template.j2> [-d <data.csv>] | -dt <dt.yml>) [-g <vars.yml>] [-ed <exts dir>] [-o <output file>] [-od <output dir>] [-m] [-q]'
 
     parser = __ArgumentParser(add_help=False, usage='%(prog)s ' + jinjafx_usage)
-    group_ex = parser.add_mutually_exclusive_group(required=True)
-    group_ex.add_argument('-dt', metavar='<dt.yml>', type=argparse.FileType('r'))
-    group_ex.add_argument('-t', metavar='<template.j2>', type=argparse.FileType('r'))
+    #group_ex = parser.add_mutually_exclusive_group(required=True)
+    #group_ex.add_argument('-dt', metavar='<dt.yml>', type=argparse.FileType('r'))
+    #group_ex.add_argument('-t', metavar='<template.j2>', type=argparse.FileType('r'))
+    parser.add_argument('-t', metavar='<template.j2>', type=argparse.FileType('r'), required=True)
     parser.add_argument('-d', metavar='<data.csv>', type=argparse.FileType('r'))
     parser.add_argument('-g', metavar='<vars.yml>', type=argparse.FileType('r'), action='append')
     parser.add_argument('-ed', metavar='<exts dir>', type=str, action='append', default=[])
@@ -82,8 +43,8 @@ def main():
     parser.add_argument('-q', action='store_true')
     args = parser.parse_args()
 
-    if args.dt is not None and args.d is not None:
-      parser.error("argument -d: not allowed with argument -dt")
+    #if args.dt is not None and args.d is not None:
+    #  parser.error("argument -d: not allowed with argument -dt")
 
     if args.m is True and args.g is None:
       parser.error("argument -m: only allowed with argument -g")
@@ -94,7 +55,7 @@ def main():
     data = None
     vault = [ None ]
     gvars = {}
-    dt = {}
+    #dt = {}
 
     def decrypt_vault(string):
       if string.startswith('$ANSIBLE_VAULT;'):
@@ -123,6 +84,8 @@ def main():
     def yaml_vault_tag(loader, node):
       return decrypt_vault(node.value)
 
+    yaml.add_constructor('!vault', yaml_vault_tag, yaml.SafeLoader)
+
     def merge(dst, src):
       for key in src:
         if key in dst:
@@ -140,24 +103,22 @@ def main():
 
       return dst
 
-    yaml.add_constructor('!vault', yaml_vault_tag, yaml.SafeLoader)
+    #if args.dt is not None:
+    #  with open(args.dt.name) as f:
+    #    dt.update(yaml.load(f.read(), Loader=yaml.SafeLoader)['dt'])
+    #    args.t = dt['template']
 
-    if args.dt is not None:
-      with open(args.dt.name) as f:
-        dt.update(yaml.load(f.read(), Loader=yaml.SafeLoader)['dt'])
-        args.t = dt['template']
+    #    if 'datasets' in dt:
+    #      raise Exception('invalid datatemplate format')
 
-        if 'datasets' in dt:
-          raise Exception('datasets aren\'t supported at present')
+    #    else:
+    #      if 'data' in dt:
+    #        data = dt['data']
 
-        else:
-          if 'data' in dt:
-            data = dt['data']
-
-          if 'vars' in dt:
-            gyaml = decrypt_vault(dt['vars'])
-            if gyaml:
-              gvars.update(yaml.load(gyaml, Loader=yaml.SafeLoader))
+    #      if 'vars' in dt:
+    #        gyaml = decrypt_vault(dt['vars'])
+    #        if gyaml:
+    #          gvars.update(yaml.load(gyaml, Loader=yaml.SafeLoader))
 
     if args.d is not None:
       with open(args.d.name) as f:
@@ -214,11 +175,7 @@ def main():
 
       gvars['jinjafx_input'] = jinjafx_input
 
-    if import_filters() > 0:
-      print()
-
-    base = os.path.abspath(os.path.dirname(__file__))
-    args.ed = [base + '/extensions', os.getcwd(), os.getenv('HOME') + '/.jinjafx'] + args.ed
+    args.ed = [os.getcwd(), os.getenv('HOME') + '/.jinjafx'] + args.ed
     outputs = JinjaFx().jinjafx(args.t, data, gvars, args.o, args.ed)
     ocount = 0
 
@@ -267,7 +224,7 @@ def main():
 
   except Exception as e:
     tb = traceback.format_exc()
-    match = re.search(r'[\s\S]*File "(.+)", line ([0-9]+), in.*template', tb, re.IGNORECASE)
+    match = re.search(r'^[ \t]*File "(.+)", line ([0-9]+), in template$', tb, re.IGNORECASE | re.MULTILINE)
     if match:
       print('error[' + match.group(1) + ':' + match.group(2) + ']: ' + type(e).__name__ + ': ' + str(e), file=sys.stderr)
     else:
@@ -276,6 +233,14 @@ def main():
     sys.exit(-2)
 
 
+def __format_bytes(b):
+  for u in [ '', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y' ]:
+    if b >= 1000:
+      b /= 1000
+    else:
+      return '{:.2f}'.format(b).rstrip('0').rstrip('.') + u + 'B'
+
+
 class __ArgumentParser(argparse.ArgumentParser):
   def error(self, message):
     if '-q' not in sys.argv:
@@ -296,151 +261,154 @@ class JinjaFx():
     delim = None
     rowkey = 1
     int_indices = []
-    
-    if isinstance(data, bytes):
-      data = data.decode('utf-8')
 
-    if data is not None and len(data.strip()) > 0:
-      jinjafx_filter = {}
+    if not isinstance(template, str) and not isinstance(template, io.IOBase):
+      raise TypeError('template must be of type str or type FileType')
 
-      for l in data.splitlines():
-        if len(l.strip()) > 0 and not re.match(r'^[ \t]*#', l):
-          if len(self.__g_datarows) == 0:
-            if l.count(',') > l.count('\t'):
-              delim = r'[ \t]*,[ \t]*'
-              schars = ' \t'
-            else:
-              delim = r' *\t *'
-              schars = ' '
-
-            fields = re.split(delim, re.sub('(?:' + delim + ')+$', '', l.strip(schars)))
-            fields = [re.sub(r'^(["\'])(.*)\1$', r'\2', f) for f in fields]
-
-            for i in range(len(fields)):
-              if fields[i].lower().endswith(':int'):
-                int_indices.append(i + 1)
-                fields[i] = fields[i][:-4]
-
-              if 'jinjafx_adjust_headers' in gvars:
-                jinjafx_adjust_headers = str(gvars['jinjafx_adjust_headers']).strip().lower()
-
-                if jinjafx_adjust_headers == 'yes':
-                  fields[i] = re.sub(r'[^A-Z0-9_]', '', fields[i], flags=re.UNICODE | re.IGNORECASE)
-
-                elif jinjafx_adjust_headers == 'upper':
-                  fields[i] = re.sub(r'[^A-Z0-9_]', '', fields[i].upper(), flags=re.UNICODE | re.IGNORECASE)
-
-                elif jinjafx_adjust_headers == 'lower':
-                  fields[i] = re.sub(r'[^A-Z0-9_]', '', fields[i].lower(), flags=re.UNICODE | re.IGNORECASE)
-
-                elif jinjafx_adjust_headers != 'no':
-                  raise Exception('invalid value specified for \'jinjafx_adjust_headers\' - must be \'yes\', \'no\', \'upper\' or \'lower\'')
-              
-              if fields[i] == '':
-                raise Exception('empty header field detected at column position ' + str(i + 1))
-
-              elif not re.match(r'^[A-Z_][A-Z0-9_]*$', fields[i], re.IGNORECASE):
-                raise Exception('header field at column position ' + str(i + 1) + ' contains invalid characters')
-
-            if len(set(fields)) != len(fields):
-              raise Exception('duplicate header field detected in data')
-
-            else:
-              self.__g_datarows.append(fields)
-
-            if 'jinjafx_filter' in gvars and len(gvars['jinjafx_filter']) > 0:
-              for field in gvars['jinjafx_filter']:
-                jinjafx_filter[self.__g_datarows[0].index(field) + 1] = gvars['jinjafx_filter'][field]
-
-          else:
-            gcount = 1
-            fields = []
-
-            for f in re.split(delim, l.strip(schars)):
-              delta = 0
-
-              for m in re.finditer(r'(?<!\\)\((.+?)(?<!\\)\)', f):
-                if not re.search(r'(?<!\\)\|', m.group(1)):
-                  if not re.search(r'\\' + str(gcount), l):
-                    if re.search(r'\\[0-9]+', l):
-                      raise Exception('parenthesis in row ' + str(rowkey) + ' at \'' + str(m.group(0)) + '\' should be escaped or removed')
-
-                    else:
-                      f = f[:m.start() + delta] + '\\(' + m.group(1) + '\\)' + f[m.end() + delta:]
-                      delta += 2
-
-                gcount += 1
-
-              fields.append(re.sub(r'^(["\'])(.*)\1$', r'\2', f))
-
-            n = len(self.__g_datarows[0])
-            fields = [list(map(self.__jfx_expand, fields[:n] + [''] * (n - len(fields)), [True] * n))]
-
-            # recm = r'(?<!\\){[ \t]*([0-9]+):([0-9]+)[ \t]*(?<!\\)}' # TODO: REMOVE PAD
-            recm = r'(?<!\\){[ \t]*([0-9]+):([0-9]+)(?::([0-9]+))?[ \t]*(?<!\\)}'
-
-            row = 0
-            while row < len(fields):
-              if not isinstance(fields[row][0], int):
-                fields[row].insert(0, rowkey)
-                rowkey += 1
-
-              if any(isinstance(col[0], list) for col in fields[row][1:]):
-                for col in range(1, len(fields[row])):
-                  if isinstance(fields[row][col][0], list):
-                    for v in range(len(fields[row][col][0])):
-                      nrow = copy.deepcopy(fields[row])
-                      nrow[col] = [fields[row][col][0][v], fields[row][col][1][v]]
-                      fields.append(nrow)
-
-                    fields.pop(row)
-                    break
+    if data is not None:
+      if not isinstance(data, str):
+        raise TypeError('data must be of type str')
 
+      if len(data.strip()) > 0:
+        jinjafx_filter = {}
+  
+        for l in data.splitlines():
+          if len(l.strip()) > 0 and not re.match(r'^[ \t]*#', l):
+            if len(self.__g_datarows) == 0:
+              if l.count(',') > l.count('\t'):
+                delim = r'[ \t]*,[ \t]*'
+                schars = ' \t'
               else:
-                groups = []
-
-                for col in range(1, len(fields[row])):
-                  fields[row][col][0] = re.sub(recm, lambda m: self.__jfx_data_counter(m, fields[row][0], col, row), fields[row][col][0])
-
-                  for g in range(len(fields[row][col][1])):
-                    fields[row][col][1][g] = re.sub(recm, lambda m: self.__jfx_data_counter(m, fields[row][0], col, row), fields[row][col][1][g])
-
-                  groups.append(fields[row][col][1])
-
-                groups = dict(enumerate(sum(groups, ['\\0'])))
-
-                for col in range(1, len(fields[row])):
-                  fields[row][col] = re.sub(r'\\([0-9]+)', lambda m: groups.get(int(m.group(1)), '\\' + m.group(1)), fields[row][col][0])
-
-                  delta = 0
-                  for m in re.finditer(r'([0-9]+)(?<!\\)\%([0-9]+)', fields[row][col]):
-                    pvalue = str(int(m.group(1))).zfill(int(m.group(2)))
-                    fields[row][col] = fields[row][col][:m.start() + delta] + pvalue + fields[row][col][m.end() + delta:]
-
-                    if len(m.group(0)) > len(pvalue):
-                      delta -= len(m.group(0)) - len(pvalue)
-                    else:
-                      delta += len(pvalue) - len(m.group(0))
-
-                  fields[row][col] = re.sub(r'\\([}{%])', r'\1', fields[row][col])
-
-                  if col in int_indices:
-                    fields[row][col] = int(fields[row][col])
-
-                include_row = True
-                if len(jinjafx_filter) > 0:
-                  for index in jinjafx_filter:
-                    if not re.search(jinjafx_filter[index], fields[row][index]):
-                      include_row = False
+                delim = r' *\t *'
+                schars = ' '
+  
+              fields = re.split(delim, re.sub('(?:' + delim + ')+$', '', l.strip(schars)))
+              fields = [re.sub(r'^(["\'])(.*)\1$', r'\2', f) for f in fields]
+  
+              for i in range(len(fields)):
+                if fields[i].lower().endswith(':int'):
+                  int_indices.append(i + 1)
+                  fields[i] = fields[i][:-4]
+  
+                if 'jinjafx_adjust_headers' in gvars:
+                  jinjafx_adjust_headers = str(gvars['jinjafx_adjust_headers']).strip().lower()
+  
+                  if jinjafx_adjust_headers == 'yes':
+                    fields[i] = re.sub(r'[^A-Z0-9_]', '', fields[i], flags=re.UNICODE | re.IGNORECASE)
+  
+                  elif jinjafx_adjust_headers == 'upper':
+                    fields[i] = re.sub(r'[^A-Z0-9_]', '', fields[i].upper(), flags=re.UNICODE | re.IGNORECASE)
+  
+                  elif jinjafx_adjust_headers == 'lower':
+                    fields[i] = re.sub(r'[^A-Z0-9_]', '', fields[i].lower(), flags=re.UNICODE | re.IGNORECASE)
+  
+                  elif jinjafx_adjust_headers != 'no':
+                    raise Exception('invalid value specified for \'jinjafx_adjust_headers\' - must be \'yes\', \'no\', \'upper\' or \'lower\'')
+                
+                if fields[i] == '':
+                  raise Exception('empty header field detected at column position ' + str(i + 1))
+  
+                elif not re.match(r'^[A-Z_][A-Z0-9_]*$', fields[i], re.IGNORECASE):
+                  raise Exception('header field at column position ' + str(i + 1) + ' contains invalid characters')
+  
+              if len(set(fields)) != len(fields):
+                raise Exception('duplicate header field detected in data')
+  
+              else:
+                self.__g_datarows.append(fields)
+  
+              if 'jinjafx_filter' in gvars and len(gvars['jinjafx_filter']) > 0:
+                for field in gvars['jinjafx_filter']:
+                  jinjafx_filter[self.__g_datarows[0].index(field) + 1] = gvars['jinjafx_filter'][field]
+  
+            else:
+              gcount = 1
+              fields = []
+  
+              for f in re.split(delim, l.strip(schars)):
+                delta = 0
+  
+                for m in re.finditer(r'(?<!\\)\((.+?)(?<!\\)\)', f):
+                  if not re.search(r'(?<!\\)\|', m.group(1)):
+                    if not re.search(r'\\' + str(gcount), l):
+                      if re.search(r'\\[0-9]+', l):
+                        raise Exception('parenthesis in row ' + str(rowkey) + ' at \'' + str(m.group(0)) + '\' should be escaped or removed')
+  
+                      else:
+                        f = f[:m.start() + delta] + '\\(' + m.group(1) + '\\)' + f[m.end() + delta:]
+                        delta += 2
+  
+                  gcount += 1
+  
+                fields.append(re.sub(r'^(["\'])(.*)\1$', r'\2', f))
+  
+              n = len(self.__g_datarows[0])
+              fields = [list(map(self.__jfx_expand, fields[:n] + [''] * (n - len(fields)), [True] * n))]
+  
+              recm = r'(?<!\\){[ \t]*([0-9]+):([0-9]+)[ \t]*(?<!\\)}'
+  
+              row = 0
+              while row < len(fields):
+                if not isinstance(fields[row][0], int):
+                  fields[row].insert(0, rowkey)
+                  rowkey += 1
+  
+                if any(isinstance(col[0], list) for col in fields[row][1:]):
+                  for col in range(1, len(fields[row])):
+                    if isinstance(fields[row][col][0], list):
+                      for v in range(len(fields[row][col][0])):
+                        nrow = copy.deepcopy(fields[row])
+                        nrow[col] = [fields[row][col][0][v], fields[row][col][1][v]]
+                        fields.append(nrow)
+  
+                      fields.pop(row)
                       break
-
-                if include_row:
-                  self.__g_datarows.append(fields[row])
-
-                row += 1
-
-      if len(self.__g_datarows) <= 1:
-        raise Exception('not enough data rows - need at least two')
+  
+                else:
+                  groups = []
+  
+                  for col in range(1, len(fields[row])):
+                    fields[row][col][0] = re.sub(recm, lambda m: self.__jfx_data_counter(m, fields[row][0], col, row), fields[row][col][0])
+  
+                    for g in range(len(fields[row][col][1])):
+                      fields[row][col][1][g] = re.sub(recm, lambda m: self.__jfx_data_counter(m, fields[row][0], col, row), fields[row][col][1][g])
+  
+                    groups.append(fields[row][col][1])
+  
+                  groups = dict(enumerate(sum(groups, ['\\0'])))
+  
+                  for col in range(1, len(fields[row])):
+                    fields[row][col] = re.sub(r'\\([0-9]+)', lambda m: groups.get(int(m.group(1)), '\\' + m.group(1)), fields[row][col][0])
+  
+                    delta = 0
+                    for m in re.finditer(r'([0-9]+)(?<!\\)\%([0-9]+)', fields[row][col]):
+                      pvalue = str(int(m.group(1))).zfill(int(m.group(2)))
+                      fields[row][col] = fields[row][col][:m.start() + delta] + pvalue + fields[row][col][m.end() + delta:]
+  
+                      if len(m.group(0)) > len(pvalue):
+                        delta -= len(m.group(0)) - len(pvalue)
+                      else:
+                        delta += len(pvalue) - len(m.group(0))
+  
+                    fields[row][col] = re.sub(r'\\([}{%])', r'\1', fields[row][col])
+  
+                    if col in int_indices:
+                      fields[row][col] = int(fields[row][col])
+  
+                  include_row = True
+                  if len(jinjafx_filter) > 0:
+                    for index in jinjafx_filter:
+                      if not re.search(jinjafx_filter[index], fields[row][index]):
+                        include_row = False
+                        break
+  
+                  if include_row:
+                    self.__g_datarows.append(fields[row])
+  
+                  row += 1
+  
+        if len(self.__g_datarows) <= 1:
+          raise Exception('not enough data rows - need at least two')
 
     if 'jinjafx_sort' in gvars and len(gvars['jinjafx_sort']) > 0:
       for field in reversed(gvars['jinjafx_sort']):
@@ -458,6 +426,20 @@ class JinjaFx():
           r = True if field.startswith('-') else False
           self.__g_datarows[1:] = sorted(self.__g_datarows[1:], key=lambda n: n[self.__g_datarows[0].index(field.lstrip('+-')) + 1], reverse=r)
 
+    sys.path += [os.path.abspath(os.path.dirname(__file__)) + '/extensions'] + exts_dirs
+
+    if 'jinja2_extensions' not in gvars:
+      gvars.update({ 'jinja2_extensions': [] })
+
+    if importlib.util.find_spec('ext_jinjafx') is not None:
+      gvars['jinja2_extensions'].append('ext_jinjafx.plugin')
+
+    if importlib.util.find_spec('ext_ansible_core') is not None:
+      gvars['jinja2_extensions'].append('ext_ansible_core.plugin')
+
+    if importlib.util.find_spec('ext_ansible_ipaddr') is not None:
+      gvars['jinja2_extensions'].append('ext_ansible_ipaddr.plugin')
+
     jinja2_options = {
       'undefined': jinja2.StrictUndefined,
       'trim_blocks': True,
@@ -465,34 +447,13 @@ class JinjaFx():
       'keep_trailing_newline': True
     }
 
-    if 'jinja2_extensions' not in gvars:
-      gvars.update({ 'jinja2_extensions': [] })
-
-    gvars['jinja2_extensions'].insert(0, 'ext.jinjafx')
-    sys.path += exts_dirs
-
-    if isinstance(template, bytes) or isinstance(template, str):
+    if isinstance(template, str):
       env = jinja2.Environment(extensions=gvars['jinja2_extensions'], **jinja2_options)
-      [env.filters.update(f) for f in jinja2_filters]
-      if isinstance(template, bytes):
-        template = env.from_string(template.decode('utf-8'))
-      else:
-        template = env.from_string(template)
+      template = env.from_string(template)
     else:
       env = jinja2.Environment(extensions=gvars['jinja2_extensions'], loader=jinja2.FileSystemLoader(os.path.dirname(template.name)), **jinja2_options)
-      [env.filters.update(f) for f in jinja2_filters]
       template = env.get_template(os.path.basename(template.name))
 
-    env.tests.update({
-      'regex': self.__jfx_regex,
-      'match': self.__jfx_match,
-      'search': self.__jfx_search
-    })
-
-    env.globals.update({
-      'lookup': self.__jfx_lookup
-    })
-
     env.globals.update({ 'jinjafx': {
       'version': __version__,
       'jinja2_version': jinja2.__version__,
@@ -506,10 +467,11 @@ class JinjaFx():
       'setg': self.__jfx_setg,
       'getg': self.__jfx_getg,
       'now': self.__jfx_now,
-#      'nslookup': self.__jfx_nslookup,
       'rows': max([0, len(self.__g_datarows) - 1]),
       'data': [r[1:] if isinstance(r[0], int) else r for r in self.__g_datarows]
-    }})
+    },
+      'lookup': self.__jfx_lookup
+    })
 
     if len(gvars) > 0:
       env.globals.update(gvars)
@@ -583,6 +545,14 @@ class JinjaFx():
     return outputs
 
 
+  def __find_re_match(self, o, v, default=0):
+    for rx in o:
+      if rx[0].match(v):
+        return rx[1]
+
+    return default
+
+
   def __jfx_lookup(self, method, variable, default=None):
     if method == 'vars':
       if variable in self.__g_vars:
@@ -601,21 +571,13 @@ class JinjaFx():
   def __jfx_data_counter(self, m, orow, col, row):
     start = m.group(1)
     increment = m.group(2)
-    pad = int(m.group(3)) if m.lastindex == 3 else 0 # TODO: REMOVE PAD
-
-    if pad > 0:
-      message = "padding on counters has been deprecated - please use the '%' pad operator instead"
-      if message not in self.__g_warnings:
-        self.__g_warnings.append(message)
-
     key = '_datacnt_r_' + str(orow) + '_' + str(col) + '_' + m.group()
 
     if self.__g_dict.get(key + '_' + str(row), True):
       n = self.__g_dict.get(key, int(start) - int(increment))
       self.__g_dict[key] = n + int(increment)
       self.__g_dict[key + '_' + str(row)] = False
-    # return str(self.__g_dict[key]) # TODO: REMOVE PAD
-    return str(self.__g_dict[key]).zfill(pad)
+    return str(self.__g_dict[key])
 
 
   def __jfx_expand(self, s, rg=False):
@@ -639,8 +601,7 @@ class JinjaFx():
 
       i = 0
       while i < len(pofa):
-        # m = re.search(r'(?<!\\)\{[ \t]*([0-9]+-[0-9]+):([0-9]+)[ \t]*(?<!\\)\}', pofa[i]) # TODO: REMOVE PAD
-        m = re.search(r'(?<!\\)\{[ \t]*([0-9]+-[0-9]+):([0-9]+)(?::([0-9]+))?[ \t]*(?<!\\)\}', pofa[i])
+        m = re.search(r'(?<!\\)\{[ \t]*([0-9]+-[0-9]+):([0-9]+)[ \t]*(?<!\\)\}', pofa[i])
         if m:
           mpos = groups[i][0].index(m.group())
           nob = len(re.findall(r'(?<!\\)\(', groups[i][0][:mpos]))
@@ -655,14 +616,7 @@ class JinjaFx():
           step = int(m.group(2)) if end > start else 0 - int(m.group(2))
 
           for n in range(start, end, step):
-            if m.lastindex == 3:
-              message = "padding on counters has been deprecated - please use the '%' pad operator instead"
-              if message not in self.__g_warnings:
-                self.__g_warnings.append(message)
-
-            n = str(n).zfill(int(m.group(3)) if m.lastindex == 3 else 0) # TODO: REMOVE PAD
-            # pofa.append(pofa[i][:m.start(1) - 1] + str(n) + pofa[i][m.end(m.lastindex) + 1:]) # TODO: REMOVE PAD
-            pofa.append(pofa[i][:m.start(1) - 1] + n + pofa[i][m.end(m.lastindex) + 1:])
+            pofa.append(pofa[i][:m.start(1) - 1] + str(n) + pofa[i][m.end(m.lastindex) + 1:])
 
             ngroups = list(groups[i])
             if group > 0 and group < len(ngroups):
@@ -845,57 +799,5 @@ class JinjaFx():
       return str(datetime.datetime.utcnow().astimezone(tz))
 
 
-#  def __jfx_nslookup(self, v, family=46):
-#    try:
-#      if re.match(r'^(?:[0-9a-f:]+:+)+[0-9a-f]+$', v, re.I): # IPv6
-#        return [socket.getnameinfo((v, 0), socket.NI_NAMEREQD)[0]]
-#
-#      elif re.match(r'^(?:[0-9]+\.){3}[0-9]+$', v): # IPv4
-#        return [socket.getnameinfo((v, 0), socket.NI_NAMEREQD)[0]]
-#
-#      else:
-#        if int(family) == 46:
-#          s = socket.getaddrinfo(v, 0, 0, socket.SOCK_STREAM)
-#          return [e[4][0] for e in s]
-#        elif int(family) == 4:
-#          s = socket.getaddrinfo(v, 0, socket.AF_INET, socket.SOCK_STREAM)
-#          return [e[4][0] for e in s]
-#        elif int(family) == 6:
-#          s = socket.getaddrinfo(v, 0, socket.AF_INET6, socket.SOCK_STREAM)
-#          return [e[4][0] for e in s]
-#
-#    except:
-#      pass
-#
-#    return None
-
-
-  def __jfx_regex(self, value='', pattern='', ignorecase=False, multiline=False, match_type='search', flags=0):
-    if ignorecase:
-      flags |= re.I
-
-    if multiline:
-      flags |= re.M
-
-    _re = re.compile(pattern, flags=flags)
-    return bool(getattr(_re, match_type, 'search')(value))
-
-
-  def __jfx_match(self, value, pattern='', ignorecase=False, multiline=False):
-    return self.__jfx_regex(value, pattern, ignorecase, multiline, 'match')
-
-
-  def __jfx_search(self, value, pattern='', ignorecase=False, multiline=False):
-    return self.__jfx_regex(value, pattern, ignorecase, multiline, 'search')
-
-
-  def __find_re_match(self, o, v, default=0):
-    for rx in o:
-      if rx[0].match(v):
-        return rx[1]
-
-    return default
-
-
 if __name__ == '__main__':
   main()
